// C Source File

/* --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AGENDA V4.2  oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Main File ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Date:         July 2003   ooooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------
// -- internal Version  */ #define I_VERSION "4_2_0" //----------------------
// -- Last modification ** 15. X.2004 --- 13:57 -----------------------------
// -- Built with TIGCC  ** 0.95 Beta 19 -------------------------------------
// --------------------------------------------------------------------------
// -- GCC-Switches ******* -Os -Wall -W -Wwrite-strings *********************
// --------------- ******* -Wno-unused -mregparm ****************************
// --------------- *******   |- because of own event functions which ********
// --------------- *******      sometimes have unused function parameters ***
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// -- Program options **** Reloc Format * FLINE, internal FLINE-Emu.tor *****
// ------------------ **** BSS Format *** Kernel ****************************
// --------------------------------------------------------------------------


#include "IncludeAll.h"
#include "Icons.h"
#include "Global.h"


// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int GetPassword ()
{
	unsigned int	Flags;
	int				Ret			= ER_SUCCESS;
	char			Password[9];
	
	Flags = GetFlags ();
	if (Flags & FLAG_PASSWORD)
	{
		Ret = PasswordDlg ( DLG_PW_TITLE, DLG_ENTER_PW, Password);
		if (Ret == OK)
		{
			EncodeData (Password, ENCODE_KEY_P, 8);
			if ( !memcmp (Password, &gr_Agenda->DataFile.Header->Password, 8) ) Ret = ER_SUCCESS;
			else Ret = ER_WRONG_PW;
		}
	}
	
	return (Ret);
}

// --------------------------------------------------------------------------

void InitAgenda ()
{
	memset (gr_Agenda, 0, sizeof (TAgenda));

	gr_Agenda->OldInitKeyDelay = OSInitKeyInitDelay (KEY_INIT_DELAY);
	gr_Agenda->OldBetweenKeyDelay = OSInitBetweenKeyDelay (BETWEEN_KEY_DELAY);
	gr_Agenda->DataFile.Path = GetSymnamePtr (gr_Agenda->DataFile.PathBuf, NULL, DATA_FILE_NAME);
	gr_Agenda->BackupFilePath = GetSymnamePtr (gr_Agenda->BFPathBuffer, NULL, BACKUP_FILE_NAME);
	gr_Agenda->keyCombo = 0;
	gr_Agenda->TurnedOff = 0;
	gr_Agenda->OldInt6 = GetIntVec (AUTO_INT_6);

	

	// ---------------
	// if AMS >= 2.07
	// ---------------
	if ( (TIOS_entries > 0x5EF) && IsClockOn() )
	{
		gr_Agenda->UseClock = 1;
	}
	
	#ifdef FORCE_USE_CLOCK
		gr_Agenda->UseClock = 1;
		#warning "FORCE_USE_CLOCK is used!"
	#endif
}

// --------------------------------------------------------------------------
// --- Launch Agenda --------------------------------------------------------
// --------------------------------------------------------------------------

void StartAgenda ()
{
	int				Ret 				= ER_SUCCESS;
	int				OpenedCorrectly		= 0;
	int				ExitCode;
	HANDLE			Screen;
	TAgenda			Agenda;
	
	gr_Agenda = &Agenda;					// global register Pointer %a3
	g_Agenda = &Agenda;						// global pointer to Agenda for using in CALLBACKS

	InitAgenda ();
	
	Ret = SaveScreen (&Screen);
	
	if (HeapAvail() < 5*1024) Ret = ER_MALLOC;			

	if (Ret == ER_SUCCESS)
	{
		Ret = OpenDataFile (); 
		if (Ret == ER_DATAFILE_ARCHIVED) Ret = MakeTwin ();
	}
	if (Ret == ER_SUCCESS)
	{	
		Ret = GetPassword ();
		if (Ret == ER_SUCCESS)
		{
			OpenedCorrectly = 1;
			if (gr_Agenda->isTwin) MsgBox ( MSG_TITLE_HINT, MSG_LOSEDATA, BT_OK, BT_NONE );

			if (gr_Agenda->UseClock) OSVRegisterTimer (1, 20, CBShowClock);
			InitScreen ();
			ExitCode = TaskMenu ();
			gr_Agenda->DataFile.Header->LastTask = ExitCode;
		}
		CloseDataFile();
	}
	ShowErrors (Ret);
	
	OSVFreeTimer (1);
	RestoreScreen (&Screen);
	
	if (OpenedCorrectly)
	{
		ManageArchivedDataFile ();
		ManageAutoBackup ();
	}
		
	OSClearBreak();

	OSInitKeyInitDelay (gr_Agenda->OldInitKeyDelay);
	OSInitBetweenKeyDelay (gr_Agenda->OldBetweenKeyDelay);

	if (gr_Agenda->OffOnExit) off();

}

// --------------------------------------------------------------------------

INT_HANDLER		OldTrap4;

DEFINE_INT_HANDLER(IntHandlerTrap4)
{
	int				res			= ER_SUCCESS;
	int				usePW;
	HANDLE			handle;
	void			*oldA5;
	TAgenda			*oldA3;
	
	// --- restore a5 with jump table base (OPTIMIZE_ROM_CALLS)
	// --- and restore a3, the global Agenda struct pointer
	oldA3 = gr_Agenda;
	gr_Agenda = g_Agenda;

	#ifdef USE_OPT_ROM_CALLS
		oldA5 = __jmp_tbl;
		__jmp_tbl = *(void **)0xC8;
	#endif
	
	OSVFreeTimer (1);

	gr_Agenda->TurnedOff = 1;
	stIdle();

	usePW = (GetFlags() & FLAG_PASSWORD) && !gr_Agenda->OffOnExit;

	if (usePW)
	{
		SaveScreen (&handle);
		fillDisplay();
	}
	
	ExecuteHandler (OldTrap4);

	if (usePW)
	{
		do
		{
			res = GetPassword ();
			if (res == ER_WRONG_PW || res == CANCEL) ExecuteHandler (OldTrap4); 
		}
		while (res == ER_WRONG_PW || res == CANCEL);

		RestoreScreen (&handle);
	}

	if (gr_Agenda->UseClock && !gr_Agenda->OffOnExit) OSVRegisterTimer (1, 20, CBShowClock);

	// --- some tsr's, which itselfs have their Trap 4 handlers change the vector
	// --- back to their own handler. (e.g. fastkbd)
	// --- Therefore, make sure that Agenda's Trap 4 handler is installed again.
	SetIntVec (INT_VEC_OFF, IntHandlerTrap4);

	// --- restore original a5 and a3 with the values they had on entrance

	#ifdef USE_OPT_ROM_CALLS
		__jmp_tbl = oldA5;
	#endif

	gr_Agenda = oldA3;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- MAIN -----------------------------------------------------------------
// --------------------------------------------------------------------------

void _main(void)
{
	char			stText[80];

	OldTrap4 = GetIntVec (INT_VEC_OFF);	
	SetIntVec (INT_VEC_OFF, IntHandlerTrap4);

	TRY
		#ifndef MAKE_OFFICIAL_RELEASE
			sprintf (stText, "%s - unofficial release - build %s",CL_INFO_SHORT, I_VERSION);
		#else
			sprintf (stText, "%s",CL_INFO);
		#endif
			ST_helpMsg (stText);

		StartAgenda ();

		#ifndef MAKE_OFFICIAL_RELEASE
			sprintf (stText, "%s - unofficial release - build %s",CL_INFO_SHORT, I_VERSION);
		#else
			sprintf (stText, "%s",CL_INFO);
		#endif
			ST_helpMsg (stText);
	ONERR
		ERD_dialog (errCode, 0);
	ENDTRY

	SetIntVec (INT_VEC_OFF, OldTrap4);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------


