// C Source File

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AGENDA V4.2  oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------


#include "IncludeAll.h"


// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void InitEmptyDataFile (TDataFile *DataFile)
{
	char		*DFPtr = (char *)&DataFile->Size;
	
	memset (DFPtr, 0, sizeof(TDataFile)-1);
	DataFile->Size = sizeof(TDataFile)-3;
	DataFile->Version = DF_VERSION;
	DataFile->Flags = FLAGS_INIT;
	
	// optimize: replace by offsettof
	DataFile->AgendaPtr = (int)((char *)&DataFile->AgendaData -DFPtr);
	DataFile->AutofieldPtr = (int)((char *)&DataFile->AutofieldData -DFPtr);
	DataFile->AddressbookPtr = (int)((char *)&DataFile->AddressbookData -DFPtr);
	DataFile->NotesPtr = (int)((char *)&DataFile->NotesData -DFPtr);
	DataFile->INetLinksPtr = (int)((char *)&DataFile->INetLinksData -DFPtr);

	DataFile->AgendaSize = 2;
	DataFile->AutofieldSize = 2;
	DataFile->AddressbookSize = 2;
	DataFile->NotesSize = 2;
	DataFile->INetLinksSize = 2;

	sprintf (DataFile->Type, "data");
	DataFile->Tag = OTH_TAG;		
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int CreateEmptyDataFile ()
{
	int			Ret			= ER_SUCCESS;
	HSym		HSymF;
	SYM_ENTRY	*SymEntry	= NULL;
	
	HSymF = SymAdd (gr_Agenda->DataFile.Path);
	if (!HSymF.folder) Ret = ER_CREATE_DATAFILE;
	if (Ret == ER_SUCCESS)
	{
		SymEntry = DerefSym (HSymF); 
		SymEntry->handle = HeapAlloc (sizeof(TDataFile)-1);
		if (SymEntry->handle == H_NULL) Ret = ER_MALLOC;
	}
	if (Ret == ER_SUCCESS)
	{
		SymEntry->flags.bits.hidden = 1;
		gr_Agenda->DataFile.handle = SymEntry->handle;
		gr_Agenda->DataFile.DataPtr = HeapDeref (SymEntry->handle);
		gr_Agenda->DataFile.Header = (TDataFile *)gr_Agenda->DataFile.DataPtr;

		InitEmptyDataFile (gr_Agenda->DataFile.Header);
	}
	
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Opens or creates the datafile. Locks its handle ----------------------
// --------------------------------------------------------------------------

int OpenDataFile ()
{
	int				Ret				= ER_SUCCESS;
	unsigned int 	Flags;
	SYM_ENTRY		*SymEntry		= NULL;
	HSYM			HSymF;
	
	HSymF = SymFind (gr_Agenda->DataFile.Path);
	
	if (HSymF.folder)
	{
		SymEntry = DerefSym (HSymF);
		if (SymEntry->flags.bits.archived) Ret = ER_DATAFILE_ARCHIVED;  
		else
		{
			gr_Agenda->DataFile.DataPtr = HeapDeref (SymEntry->handle);
			gr_Agenda->DataFile.Header = (TDataFile *)gr_Agenda->DataFile.DataPtr;
			if ( gr_Agenda->DataFile.Header->Version != DF_VERSION ) Ret = ER_WRONG_DF_VERSION;
			else
			{
				gr_Agenda->DataFile.handle = SymEntry->handle;
				SymEntry->flags.bits.hidden = 1;
			}
		}
	}
	else
	{
		Ret = CreateEmptyDataFile ();
	}

	if (Ret == ER_SUCCESS)
	{
		HeapLock (gr_Agenda->DataFile.handle);

		Flags = GetFlags();
		if (Flags & FLAG_ENCODED)
		{
			EncodeFile ();
			SetFlags (Flags & ~FLAG_ENCODED);
			
		}
	}
	return (Ret);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Encrypt the file and unlocks the handle ------------------------------
// --------------------------------------------------------------------------

void CloseDataFile ()
{
	unsigned int	Flags;
	
#ifndef DONT_ENCODE_ONCLOSE
	Flags = GetFlags();
	if (!(Flags & FLAG_ENCODED))
	{
		SetFlags (Flags | FLAG_ENCODED);
		EncodeFile ();
	}
#else
	#warning "Datafile won't be encoded on close!"	
#endif	

	PrepareAutoBackup ();
	
	DerefSym (SymFind (gr_Agenda->DataFile.Path))->flags.bits.hidden = 0;
	HeapUnlock (gr_Agenda->DataFile.handle);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Encodes the content of the whole datafile  ---------------------------
// --------------------------------------------------------------------------

void EncodeFile ()
{
	TFile		File = gr_Agenda->DataFile;
	int 		dataptr = offsetof (TDataFile, AgendaData);

	EncodeData (File.DataPtr + dataptr, ENCODE_KEY_D, File.Header->Size +2 -N_TERM_BYTES -dataptr );
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void ReDerefDataFile ()
{
	gr_Agenda->DataFile.DataPtr = HeapDeref (gr_Agenda->DataFile.handle);
	gr_Agenda->DataFile.Header = (TDataFile *)gr_Agenda->DataFile.DataPtr;

}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Reallocates the datafile with NewSize --------------------------------
// --- if NewSize is larger than the old, the file will be enlarged, else it
// --- will be reduced.
// --- Pos points to the place, where the data will be inserted, or where
// --- data should be deleted.
// --------------------------------------------------------------------------

int ReallocDataFile (char *Pos, unsigned long NewSize, char **NewPos)
{
	int				Ret			= ER_SUCCESS;
	TFile			*File		= &gr_Agenda->DataFile;
	unsigned int	oldSize;

	oldSize = File->Header->Size + 2;
	HeapUnlock (File->handle);
	Ret = ReallocHandle (File->handle, oldSize, NewSize, Pos, NewPos);
	HeapLock (File->handle);
	ReDerefDataFile ();
	if (Ret == ER_SUCCESS)
	{
		File->Header->Size = NewSize - 2;
		gr_Agenda->Changed = 1;
	}
	
	return (Ret);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Flag-functions: flag is the 3rd and 4th byte of agnddata -------------
// --------------------------------------------------------------------------

unsigned int GetFlags ()
{
	return (gr_Agenda->DataFile.Header->Flags);
}

unsigned int SetFlags (unsigned int Flags)
{
	unsigned int	OldFlags;
	OldFlags = GetFlags();
	gr_Agenda->DataFile.Header->Flags = Flags;
	return OldFlags;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int MakeBackup ()
{
	int				Ret		= ER_SUCCESS;
	char			*pBackup;
	unsigned int	Flags;
	unsigned int	Size;
	TDataFile		*BackupFile;
	int 			dataptr = offsetof (TDataFile, AgendaData);

	Ret = DelArchivedSym (gr_Agenda->BackupFilePath);
	if ( (Ret == ER_SUCCESS) || (Ret == ER_SYM_NOT_FOUND) )
	{
		gr_Agenda->Changed = 0;
		gr_Agenda->DataFile.Header->nStarts = 0;
		HeapUnlock (gr_Agenda->DataFile.handle);
		Ret = CopySym (gr_Agenda->BackupFilePath, gr_Agenda->DataFile.Path, &pBackup);
		HeapLock (gr_Agenda->DataFile.handle);
		ReDerefDataFile ();
	}
	if (Ret == ER_SUCCESS)
	{
		BackupFile = (TDataFile *)pBackup;
		Flags = BackupFile->Flags;
		if (!(Flags & FLAG_ENCODED))
		{
			Size = peek_w (pBackup)+2;
			BackupFile->Flags = Flags | FLAG_ENCODED;
			EncodeData (pBackup + dataptr, ENCODE_KEY_D, Size -dataptr -N_TERM_BYTES);
		}
		if (!EM_moveSymToExtMem (gr_Agenda->BackupFilePath, HS_NULL)) Ret = ER_ARCHIVE_BF;
	}	
	
	return Ret;	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------


int RestoreBackup ()
{
	int				Ret		= ER_SUCCESS;
	unsigned int	szBackup;
	char			*pBackup;
	HANDLE			handle;
	char			Password[9];
	TDataFile		*BackupFile;
	unsigned int	Flags;

	Ret = GetSymAddr (gr_Agenda->BackupFilePath, &pBackup);
 	if (Ret == ER_SUCCESS)
 	{
		BackupFile = (TDataFile *)pBackup;
		if ( BackupFile->Version != DF_VERSION ) Ret = ER_WRONG_DF_VERSION;
		if (Ret == ER_SUCCESS)
		{
			if (peek (BackupFile->Password) != 0)
			{
				Ret = PasswordDlg (	DLG_PW_TITLE, DLG_ENTER_PW3, Password );
				if (Ret == OK)
				{
					Ret = ER_SUCCESS;
					EncodeData (Password, ENCODE_KEY_P, 8); 
					if (memcmp (BackupFile->Password, Password, 8)) Ret = ER_WRONG_PW;
				}
			}
		}
 	}
	if (Ret == ER_SUCCESS)
	{
		GetSymSize (gr_Agenda->BackupFilePath, &szBackup);
		HeapUnlock (gr_Agenda->DataFile.handle);
		handle = HeapRealloc (gr_Agenda->DataFile.handle, szBackup);
		HeapLock (gr_Agenda->DataFile.handle);
		ReDerefDataFile ();
		if (!handle) Ret = ER_MALLOC;
	}
	if (Ret == ER_SUCCESS)
	{
		GetSymAddr (gr_Agenda->BackupFilePath, &pBackup);
		memmove (gr_Agenda->DataFile.DataPtr, pBackup, szBackup);
		gr_Agenda->DataFile.Header->Size = szBackup-2;

		Flags = GetFlags();
		if (Flags & FLAG_ENCODED)
		{
			EncodeFile ();
			SetFlags (Flags & ~FLAG_ENCODED);
		}
		gr_Agenda->Changed = 0;
	}
	
	if (Ret == ER_SYM_NOT_FOUND) Ret = ER_NO_BACKUPFILE;

	
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void ManageArchivedDataFile ()
{
	int				Ret 			= ER_SUCCESS;
	
	if (gr_Agenda->isTwin)
	{
		if ( !SymDelTwin ( DerefSym ( SymFind (gr_Agenda->DataFile.Path))) ) Ret = ER_DEL_TWIN;
	}
	ShowErrors (Ret);	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void PrepareAutoBackup ()
{
	unsigned int	FlagsAB		= (GetFlags () & FLAGS_AB) >> FLAG_AB_OFFSET;
	unsigned int	*nStarts	= &gr_Agenda->DataFile.Header->nStarts;
	
	if (!gr_Agenda->isTwin && FlagsAB && (gr_Agenda->Changed)) (*nStarts)++;
	gr_Agenda->nStarts = *nStarts;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void ManageAutoBackup ()
{
	int				Ret			= ER_SUCCESS;
	unsigned int	FlagsAB		= (GetFlags () & FLAGS_AB) >> FLAG_AB_OFFSET;
	unsigned int	savedNum	= FlagsAB * 5;
	
	if (!gr_Agenda->isTwin)
	{
		if (FlagsAB)
		{
			if (gr_Agenda->nStarts >= savedNum)
			{
				if (MsgBox (CL_INFO_SHORT, MSG_AUTOBACKUP, BT_OK, BT_CANCEL) == OK) Ret = MakeBackup ();
			}
		}
	}

	ShowErrors (Ret);
}

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int MakeTwin ()
{
	int		Ret = ER_SUCCESS;

	if ( EM_twinSymFromExtMem ( NULL, SymFind (gr_Agenda->DataFile.Path)).folder )
	{
		Ret = OpenDataFile ();
		gr_Agenda->isTwin = 1;
	}
	else Ret = ER_MAKE_TWIN;
	
	return (Ret);
}

// --------------------------------------------------------------------------

void browseEntries (int (* callback)(TEntryHeader *, void *, int), void *ptr1, int int1)
{
	int				ret				= 0;
	TEntryHeader	*header;
	char			*entries[4];
	int				i;
	
	entries[0] = AgendaGetBasePtr();
	entries[1] = ABookGetBasePtr();
	entries[2] = NotepadGetBasePtr();
	entries[3] = LinksGetBasePtr();
	
	for (i = 0; i < 4; i++)
	{
		header = (TEntryHeader *)entries[i];
		while (!ret && header->Flags != FLAG_ET_DATAEND)
		{
			// --- callback must return 0 to continue, otherwise browsing will abort
			ret = callback (header, ptr1, int1);
			header = (TEntryHeader *)((char *)header + header->Size);
		}
		
	}
}

void setLastState (int Task, const unsigned int *group, const unsigned int *entry, const unsigned int *mode)
{
	
	TDataFile		*file	= gr_Agenda->DataFile.Header;
	
	if (Task == TASK_ADDRESSBOOK)
	{
		if (group != NULL) file->LastABGroup = *group;
		if (entry != NULL) file->LastABEntry = *entry;
		if (mode != NULL) file->LastABMode = *mode;
	}
	else if (Task == TASK_NOTEPAD)
	{
		if (group != NULL) file->LastNPGroup = *group;
		if (entry != NULL) file->LastNPEntry = *entry;
		if (mode != NULL) file->LastNPMode = *mode;
	}
	else if (Task == TASK_INET_LINKS)
	{
		if (group != NULL) file->LastWLGroup = *group;
		if (entry != NULL) file->LastWLEntry = *entry;
		if (mode != NULL) file->LastWLMode = *mode;
	}

}


void getLastState (int Task, unsigned int *group, unsigned int *entry, unsigned int *mode)
{
	
	TDataFile		*file	= gr_Agenda->DataFile.Header;
	
	if (Task == TASK_ADDRESSBOOK)
	{
		if (group != NULL) *group = file->LastABGroup;
		if (entry != NULL) *entry = file->LastABEntry;
		if (mode != NULL) *mode = file->LastABMode;
	}
	else if (Task == TASK_NOTEPAD)
	{
		if (group != NULL) *group = file->LastNPGroup;
		if (entry != NULL) *entry = file->LastNPEntry;
		if (mode != NULL) *mode = file->LastNPMode;
	}
	else if (Task == TASK_INET_LINKS)
	{
		if (group != NULL) *group = file->LastWLGroup;
		if (entry != NULL) *entry = file->LastWLEntry;
		if (mode != NULL) *mode = file->LastWLMode;
	}

}

void lastEditorMode (int Task, int state)
{
	unsigned int	lastMode;
	
	getLastState (Task, NULL, NULL, &lastMode);
	if (state) lastMode |= EDITOR_VIEW;
	else lastMode = lastMode & ~EDITOR_VIEW;
	setLastState (Task, NULL, NULL, &lastMode);
}

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
