// C Source File

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AGENDA V4.2  oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------


#include "IncludeAll.h"

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int convertDates (char *buf, HANDLE *handle, int new, int old, int rangeCheck)
{
	int			Ret			= ER_SUCCESS;
	HANDLE		hDates		= H_NULL;
	TDate		*date		= NULL;
	char		*pNewDates	= NULL;
	int			size		= 0;
	int			dsFlags;
	char		datebuf[20];
		
	writeDateFormat (old);
	Ret = GetDayFromStrDateEx (buf, &hDates, rangeCheck);

	if (Ret == ER_SUCCESS)
	{
		writeDateFormat (new);
		date = HeapDeref (hDates);
		if ( (*handle = HeapAlloc (6)) == H_NULL) Ret = ER_MALLOC;
	}
	
	while (Ret == ER_SUCCESS && (*(int *)date != -1))
	{
		if (date->Year != 0) dsFlags = FLAG_USE_YEAR | FLAG_LONG_YEAR;
		else dsFlags = 0;

		if (date->MonthDay != 0 && date->Month != 0)
		{
			getDateString (0, date, datebuf, dsFlags, NULL);				
			if (HeapRealloc (*handle, size + strlen (datebuf) +4) == H_NULL) Ret = ER_MALLOC;
			else
			{
				strcat (datebuf, "; ");
				pNewDates = HeapDeref (*handle);
				strcpy (pNewDates + size, datebuf);
				size += strlen (datebuf);
			}
		}
		date++;
	}
	
	if (Ret == ER_SUCCESS)
	{
		if (size) poke (pNewDates + size -2, 0);
	}
	
	HeapFreeHNULL (&hDates);
	
	return Ret;
}


int convertAddressbookDates (int new, int old)
{
	int				Ret			= ER_SUCCESS;
	TEntryHeader	*header 	= (TEntryHeader *)ABookGetBasePtr();
	HANDLE			hNewDates	= H_NULL;
	char			*birthday;
	TEntryType		et;
	int				groupNr		= -1;
	int				dsNr		= -1;
	HANDLE			hOldEntry[9];
	int				shortcut;
	
	ST_helpMsg (TEXT_CONVERT_DATES);
	stBusy();
	InitEntryType (&et, TASK_ADDRESSBOOK);

	while ( (header->Flags != FLAG_ET_DATAEND) && (Ret == ER_SUCCESS) )
	{
		if ( (header->Flags & FLAG_ET_MASK) == FLAG_ET_GROUP)
		{
			groupNr++;
			dsNr = -1;
		}

		if ( (header->Flags & FLAG_ET_MASK) == FLAG_ET_ENTRY)
		{
			dsNr++;
			birthday = GetStrFromListEx (&header->data, 7);
			hNewDates = H_NULL;
			Ret = convertDates (birthday, &hNewDates, new, old, 0); 

			if (Ret == ER_SUCCESS)
			{
				memset (hOldEntry, 0, sizeof(hOldEntry));
				Ret = GetDatasetInHandles (&et, groupNr, dsNr, hOldEntry, NULL);
				if (Ret == ER_SUCCESS)
				{
					shortcut = header->Shortcut;
					if ( (HeapRealloc (hOldEntry[7], strlen (HeapDeref (hNewDates))+1) == H_NULL)) Ret = ER_MALLOC;
					else
					{
						strcpy (HeapDeref (hOldEntry[7]), HeapDeref(hNewDates));
						// -- don't forget to move *header* to right pos, if file
						// -- was moved in case of heap compression!!
						Ret = AddDataset (&et, hOldEntry, groupNr, dsNr, 1, &header, shortcut);
					}
					ReleasehList (&et, hOldEntry);
				} 
			}
		}
		HeapFreeHNULL (&hNewDates);
		header = (TEntryHeader *)( ((char *)header) + header->Size);
	}

	writeDateFormat (old);
	stIdle();
	ST_eraseHelp();
	return Ret;
}

// --------------------------------------------------------------------------
// Reads the options. Returns the number according to the right pulldown
// entry in the config dialog.
// --------------------------------------------------------------------------

int readDateFormat ()
{
	int			i;
	int			ret			= 0;
	TDateFormat	options		= getDateOptions();

	for (i = 0; i < 8; i++)
	{
		if (g_optionTable[0][i] == options.order && g_optionTable[1][i] == options.delNum)
			ret = i+1;		
	}
	
	return ret; 		
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// Sets the options back to agnddata. option is the number returned from
// pulldown menu
// --------------------------------------------------------------------------

void writeDateFormat (int option)
{
	int		order		= g_optionTable[0][option-1];
	int		delimiter	= g_optionTable[1][option-1];
	int		flags		= GetFlags();
	
	order = order << FLAG_DF_OFFSET;
	delimiter = delimiter << FLAG_DL_OFFSET;
	
	flags &= ~FLAG_DATEFORMAT;
	flags &= ~FLAG_DELIMITER;
	flags |= order | delimiter;
	
	SetFlags (flags);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// fills a TDateFormat structure accoding to the configuration of date format
// in agnddata
// --------------------------------------------------------------------------

TDateFormat getDateOptions ()
{
	TDateFormat		options;
	int				flags		= GetFlags();
	char			delimiter[] = {'.', '/', '-'};
	
	options.order = (flags & FLAG_DATEFORMAT) >> FLAG_DF_OFFSET;
	options.delNum = (flags & FLAG_DELIMITER) >> FLAG_DL_OFFSET;
	options.delimiter = delimiter[options.delNum];
	
	return options;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// retrives the current day if clock enabled. If no clock, returns *Day
// --------------------------------------------------------------------------

int getToday (unsigned int *Day)
{
	int				Ret = ER_SUCCESS;
	unsigned short	year, month, day, hour, minute, second;
	TDate			Date;

	if (gr_Agenda->UseClock)
	{ 
		DateAndTime_Get (&year, &month, &day, &hour, &minute, &second);
		Date.Month = month;
		Date.Year = year;
		Date.MonthDay = day;
		*Day = GetDay (Date);	
	}
	else Ret = ER_NO_CLOCK;
		
	return Ret;
}

// --------------------------------------------------------------------------
// makes a string containing the date according to the display options
// buf must be 14 chars 
// --------------------------------------------------------------------------

void getDateString (unsigned int day, TDate *pdate, char *buf, int flags, TDateFormat *df)
{
	TDateFormat		options;
	TDate			date;
	int				first;
	int				second;
	char			dmbuf[8];
	char			year[6];
	char			dmybuf[12];
	
	if (df == NULL) options = getDateOptions();
	else memcpy (&options, df, sizeof(TDateFormat));
	
	if (pdate != NULL) memcpy (&date, pdate, sizeof (TDate));
	else GetDate (day, &date);
	
	if (options.order == DMY)
	{
		first = date.MonthDay;
		second = date.Month;	
	}
	else
	{
		first = date.Month;
		second = date.MonthDay;
	}
	sprintf (dmbuf, "%02d%c%02d", first, options.delimiter, second);
	
	if (flags & FLAG_USE_YEAR)
	{
		if (flags & FLAG_LONG_YEAR) sprintf (year, "%d", date.Year);
		else sprintf (year, "%02d", date.Year % 100);
		
		if (options.order == YMD) sprintf (dmybuf, "%s%c%s", year, options.delimiter, dmbuf);
		else sprintf (dmybuf, "%s%c%s", dmbuf, options.delimiter, year);
	}
	else sprintf (dmybuf, "%s%c", dmbuf, options.delimiter);
	
	if (flags & FLAG_USE_WEEKDAY) sprintf (buf, "%s %s", WEEK_DAYS[date.WeekDay], dmybuf); 
	else strcpy (buf, dmybuf);
			
}

// --------------------------------------------------------------------------
// --- Calculation of Date --------------------------------------------------
// --- Day		: Days since 1980 -------------------------------------------
// --- *Date 	: Pointer to TDate Structur, witch fetchs the real Date -----
// ----			: The weeknumber is'nt included. User GetDateEx instead. ----
// --------------------------------------------------------------------------

void GetDate (unsigned int Day, TDate *Date)
{
	int			n1;
	int			n2;
	int			i;
	int			*mns;
	
	Date->WeekDay = Day % 7;
	Date->Year = ((Day-1)*100L / 36525L) + 1980;
	Date->Week = 0;

	n1 = (((Day-1)*100L) % 36525L) / 100 +1;
	n2 = n1 / 32;
	if (Date->Year % 4) mns = mnsn;
	else mns = mnss;
	
	for (i = n2; i < 12; i++)
	{
		if (n1 <= mns [i+1])
		{
			Date->Month = i+1;
			Date->MonthDay = n1 - mns[i];
			break;
		}
	}
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Calculation of Date --------------------------------------------------
// --- Day		: Days since 1980 -------------------------------------------
// --- *Date 	: Pointer to TDate Structur, witch fetchs the real Date -----
// --------------------------------------------------------------------------

void GetDateEx (unsigned int Day, TDate *Date)
{
	GetDate (Day, Date);
	Date->Week = GetWeek (Day);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Calculation of the weeknumber ----------------------------------------
// --------------------------------------------------------------------------

int	GetWeek (unsigned int Day)
{
	TDate			Date;
	unsigned int	DayOfW1;
	unsigned int	DayNYW1;
	int				Week;
	
	Day -= Day % 7;
	Date.MonthDay = 4;
	Date.Month = 1;
	Date.Year = ((Day-1)*100L / 36525L) + 1980;
	DayOfW1 = GetDay (Date);
	DayOfW1 -= DayOfW1 % 7;
	Date.Year++;
	DayNYW1 = GetDay (Date);
	DayNYW1 -= DayNYW1 % 7;
	
	if (DayNYW1-Day < 7) Week = 0;
	else Week = (Day - DayOfW1) / 7;
	
	return (Week +1);	
	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Calculates the number of Days since 1680 of Date ---------------------
// --------------------------------------------------------------------------

unsigned int GetDay (TDate Date)
{
	int		*mns;
	unsigned int	ret;

	if (Date.Year % 4) mns = mnsn;
	else mns = mnss;
	ret = ((Date.Year-1980)*36525L+75) / 100 + mns[Date.Month-1] + Date.MonthDay;

	return ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int GetEasterSunday (int year)
{
	// returns the offset to 1.1. of easter sunday. 
	unsigned int	H, K, M, S, A, D, R, OG, SZ, OE;
	
	K = year / 100;
	H = ((3*K + 3) / 4);
	M = 15 + H - ((8*K+13)/25);
	S = 2 - H;
	A = year % 19;
	D = (19*A + M) % 30;
	R = D/29 + (A/11)*(D/28 - D/29);

	OG = 21 + D - R;
	SZ = 7 - ((year + year/4 + S) % 7);
	OE = 7 - ((OG - SZ) % 7);
	
	return (OG + OE - 1 + 59 + ((year % 4) == 0) );
}


int GetDayFromStrDate (char *StrDate, TDate *Date, unsigned int *Day, int rangeCheck)
{
	int			Ret			= ER_SUCCESS;
	HANDLE		Strings		= H_NULL;
	char		*ptr;
	int			n;
	int			i;
	int			Num;
	char		delimiter;
	char		dateBuffer[20];
	TDateFormat	dateOptions = getDateOptions ();
	int			orderDay	= dateOptions.order;
	int			orderMonth	= 1;
	int			orderYear	= 2;

	memcpy (dateBuffer, StrDate, sizeof(dateBuffer));
	truncTrail (dateBuffer);
	delimiter = getNumDelimiter (dateBuffer);
	if (dateOptions.order == 1) orderMonth = 0;
	if (dateOptions.order == 2) orderYear = 0;

	Num = SplitString (dateBuffer, delimiter, &Strings);

	// --- if Year is first date part and only two params are given
	if (orderYear == 0 && Num == 2)
	{
		orderDay--;
		orderMonth--;
		orderYear = -1;
	}
	
	// --- if only one param: Take it as day!
	if (Num == 1)
	{
		orderDay = 0;
		orderMonth = -1;
		orderYear = -1;
	}

	if (Strings == H_NULL) Ret = ER_MALLOC;
	else
	{		
		ptr = HeapDeref (Strings);
		for (i = 0; i < Num; i++)
		{
			n = atoi (ptr);
			ptr += strlen(ptr) +1;
			
			if (i == orderDay) if (n > 0 && n <=31) Date->MonthDay = n;

			if (i == orderMonth) if (n > 0 && n <=12) Date->Month = n;

			if (i == orderYear) 
			{
				if ( strlen (GetStrFromList (HeapDeref (Strings), i)) != 0)
				{	
					if (n >= 2000 && n< 2100) Date->Year = n;
					if (n >= 0    && n< 100 ) Date->Year = n + 2000;
					if (!rangeCheck)
					{
						if (n >= 1000 && n < 2100) Date->Year = n;
					}
				}
			}
		}

		if (Day != NULL) *Day = GetDay (*Date);
		HeapFree (Strings);
	}
	
	return (Ret);
}

// --------------------------------------------------------------------------

int GetDayFromStrDateEx (char *strDate, HANDLE *handle, int rangeCheck)
{
	int			Ret		= ER_SUCCESS;
	int			num;
	int			i;
	HANDLE		hDates	= H_NULL;
	TDate		date;
	int			size	= 2;
		
	num = SplitString (strDate, ';', &hDates);
	if (hDates == H_NULL) Ret = ER_MALLOC;
	else
	{
		if ( (*handle = HeapAlloc (2)) == H_NULL) Ret = ER_MALLOC;
		for (i = 0; i < num && Ret == ER_SUCCESS; i++)
		{
			memset (&date, 0, sizeof(TDate));
			Ret = GetDayFromStrDate (GetStrFromList (HeapDeref (hDates), i), &date, NULL, rangeCheck);
			if (Ret == ER_SUCCESS)
			{
				if ( HeapRealloc (*handle, size + sizeof (TDate)) == H_NULL) Ret = ER_MALLOC;
				else
				{
					memcpy (HeapDeref (*handle) + size -2, &date, sizeof (TDate));
					size += sizeof (TDate);
				} 
			}					
		}
		if (Ret == ER_SUCCESS)
		{
			poke_w_ex (HeapDeref (*handle) + size -2, DATE_ENDLIST);
		}
	}
	HeapFreeHNULL (&hDates);
	return Ret;
}

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void writeHashTbl (int idx, unsigned int value, int from)
{
	gr_Agenda->hashSet.hashTbl[idx] = value;
}

unsigned int readHashTbl (int idx, int from)
{
	return gr_Agenda->hashSet.hashTbl[idx];
}


int fillHashBirthdays (int year)
{
	int				Ret			= ER_SUCCESS;
	TEntryHeader	*Header;
	char			*birth;
	TDate			*date;
	int				relDay;
	int				leapYear	= ((year % 4) == 0);
	HANDLE			hDates		= H_NULL;
	
	Header = (TEntryHeader *)ABookGetBasePtr();

	while ( (Header->Flags != FLAG_ET_DATAEND) && Ret == ER_SUCCESS)
	{
		if ( (Header->Flags & FLAG_ET_MASK) == FLAG_ET_ENTRY)
		{
			birth = GetStrFromListEx (&Header->data, 7);  
			Ret = GetDayFromStrDateEx (birth, &hDates, 0);
			date = HeapDeref (hDates);
			while (Ret == ER_SUCCESS && *(int *)date != DATE_ENDLIST)
			{
				if (date->Month && date->MonthDay)
				{
					date->Year = year;
					relDay = GetDay (*date) - gr_Agenda->hashSet.firstDay;

					// optimize : make function with code below and share it with fillDayList
					writeHashTbl(relDay + HASH_TBL_OVERLAP, readHashTbl (relDay + HASH_TBL_OVERLAP, 0) | HASH_BIRTHDAY, 0);
					if (relDay < HASH_TBL_OVERLAP)
						writeHashTbl(relDay + leapYear + 365 + HASH_TBL_OVERLAP, readHashTbl (relDay + leapYear + 365 + HASH_TBL_OVERLAP, 0) | HASH_BIRTHDAY, 0);
					if (relDay > 365 - HASH_TBL_OVERLAP + leapYear)
						writeHashTbl (relDay - (365 - HASH_TBL_OVERLAP) - leapYear, readHashTbl (relDay - (365 - HASH_TBL_OVERLAP) - leapYear, 0) | HASH_BIRTHDAY, 0);

				}
				date++;
			}
			HeapFreeHNULL (&hDates);	
		}
				
		Header = (TEntryHeader *)( (char *)Header + Header->Size);
	}
	return Ret;	
}


int usePublicHoliday (int num)
{
	long	flag	= gr_Agenda->DataFile.Header->SpecialDays;
	
	flag >>= num;
	return !((int)flag & 1);
}


void fillDayList (int year, int offset, int *Days, char *IDs, int num, int lstStart)
{
	int		idx;
	int		relDay;
	int		leapYear	= ((year % 4) == 0);

	for (idx = 0; idx < num; idx++)
	{
		if (usePublicHoliday (lstStart + idx))
		{
			relDay = Days[idx] + offset;

			writeHashTbl(relDay + HASH_TBL_OVERLAP, readHashTbl (relDay + HASH_TBL_OVERLAP, 1) | IDs[idx], 1);
			if (relDay < HASH_TBL_OVERLAP)
				writeHashTbl(relDay + leapYear + 365 + HASH_TBL_OVERLAP, readHashTbl (relDay + leapYear + 365 + HASH_TBL_OVERLAP, 1) | IDs[idx], 1);
			if (relDay > 365 - HASH_TBL_OVERLAP + leapYear)
				writeHashTbl (relDay - (365 - HASH_TBL_OVERLAP) - leapYear, readHashTbl (relDay - (365 - HASH_TBL_OVERLAP) - leapYear, 1) | IDs[idx], 1);

		}
	}
}


void fillHashPublicHolidays (int year)
{
	int		easterSunday = GetEasterSunday (year);
	
	fillDayList (year, 0, absDays, absDaysIDs, 3, 0);
	fillDayList (year, easterSunday, easterRel, easterIDs, 9, 3); 
	fillDayList (year, (year % 4) == 0, relDays, relDaysIDs, 11, 12);
}


int fillHashTable (unsigned int Day, int force)
{
	int				Ret			= ER_SUCCESS;
	TDate			date;
	
	GetDate (Day, &date);

	// --- fill table only, if necessery.
	if ( (gr_Agenda->hashSet.year != date.Year) || force )
	{
		stBusy();
		ST_helpMsg (TEXT_HASH_UPDATE);
	
		date.MonthDay = 1;
		date.Month = 1;
		gr_Agenda->hashSet.firstDay = GetDay (date);	
		gr_Agenda->hashSet.year = date.Year;
		
		memset (gr_Agenda->hashSet.hashTbl, 0, sizeof (gr_Agenda->hashSet.hashTbl));
	
		Ret = fillHashBirthdays (date.Year);
		fillHashPublicHolidays (date.Year);
	
		stIdle();
		ST_eraseHelp();
	}
	
	return Ret;
}


unsigned int ReadDayWithExtraData (unsigned int Day)
{
	unsigned int	Ret;

	Ret = readHashTbl (Day - gr_Agenda->hashSet.firstDay + HASH_TBL_OVERLAP, 2);
	if (ReadDay (Day, NULL)) Ret |= NORMAL_ENTRY;
	
	return Ret;
}


int getExtraInfoCount (unsigned int Day)
{
	unsigned int	hashValue 	= ReadDayWithExtraData(Day);
	int				cnt			= 0;
	
	if (hashValue & HASH_PUBLIC_HDAY) cnt++;
	if (hashValue & HASH_AUTO_FIELD) cnt++;
	if (hashValue & HASH_BIRTHDAY) cnt++;
	
	return cnt;	
}

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
