// C Source File

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AGENDA V4.2  oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------


#include "IncludeAll.h"

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void CheckValidDay (unsigned int *Day)
{
	if (*Day > MAX_DAY) *Day = MAX_DAY;
	if (*Day < MIN_DAY) *Day = MIN_DAY;
}

// --------------------------------------------------------------------------

int ShowBirthdays (unsigned int Day)
{
	int				Ret				= ER_SUCCESS;
	unsigned int	hashValue 		= ReadDayWithExtraData (Day);
	int				idx				= 0;
	HANDLE			hBirthday;
	HANDLE			hEntryPos;
	HANDLE			hDlg;
	char			*ptr;
	
	if (hashValue & HASH_BIRTHDAY)
	{
		Ret = GetBirthdays (Day, &hBirthday, &hEntryPos);
		if (Ret == ER_SUCCESS) Ret = MakePopup (BIRTHDAY_TITLE, HeapDeref (hBirthday), 0, &hDlg); 
		if (Ret == ER_SUCCESS)
		{
			idx = PopupDo (hDlg, CENTER, CENTER, 0);
			stIdle();
			if (idx)
			{
				gr_Agenda->Exit = TASK_CHANGE;
				gr_Agenda->TaskQueue.NextTask = TASK_ADDRESSBOOK;
				
				// --- overgive the position of this entry to the message queue
				ptr = HeapDeref (hEntryPos) + (idx-1)*4;
				gr_Agenda->TaskQueue.prm0 = peek_w (ptr);
				gr_Agenda->TaskQueue.prm1 = peek_w (ptr+2);
				gr_Agenda->TaskQueue.NextMode = EDITOR_VIEW;
			}
		}
		HeapFreeHNULL (&hDlg);
		HeapFreeHNULL (&hBirthday);
		HeapFreeHNULL (&hEntryPos);
	}
	
	return Ret;
} 

int DrawExtraInfo (unsigned int Day, int eiCount, int Flags)
{
	int				Ret				= ER_SUCCESS;
	int				yPos			= 108;
	int				yLineOffset		= 0;
	unsigned int	hashValue 		= ReadDayWithExtraData (Day);
	char			birthBuf[40];
	HANDLE			hBirthdays;
	HANDLE			hEPList;
	char			*pBirthdays;
	int				nBirths;

	if (Flags == SECOND_VIEW)
	{
		yPos = 55;
		yLineOffset = 1;
	}
	
	yLineOffset += yPos + 6;
	DrawLine (9, yLineOffset - 9*eiCount, 231, yLineOffset - 9*eiCount, A_NORMAL); 
	
	if (hashValue & HASH_PUBLIC_HDAY)
	{
		printf_xy (9, yPos, "%s", GetStrFromList (PUBLIC_HOLIDAYS, (hashValue & HASH_PUBLIC_HDAY) -1) );
		yPos -= 9; 
	}
	if (hashValue & HASH_BIRTHDAY)
	{
		Ret = GetBirthdays (Day, &hBirthdays, &hEPList);
		if (Ret == ER_SUCCESS)
		{
			pBirthdays = HeapDeref (hBirthdays);
			nBirths = GetNumStrs (pBirthdays);
			strcpy (birthBuf, BIRTHDAY_TEXT);
			strncat (birthBuf, GetStrFromList (pBirthdays, 0), 17 + 4*(nBirths == 1)); 
			if (nBirths != 1) strcat (birthBuf, "; ..");
			DrawStrXY (9, yPos, birthBuf, A_NORMAL);
			if (Flags != SECOND_VIEW) DrawStrXY (208, yPos, "[\177B]", A_NORMAL);
			yPos -= 9;
		}
		HeapFreeHNULL (&hBirthdays);
		HeapFreeHNULL (&hEPList);
	}
	
	return Ret;
}

// --------------------------------------------------------------------------
// --- Open Editor. hBuf: Handle which fetches the data ---------------------
// --------------------------------------------------------------------------

int ShowEditor (HANDLE *hBuf, unsigned int Day)
{
	int				Cursor;
	int				MustCloseWin	= 0;
	int				Ret 			= ER_SUCCESS;
	int				Key;
	int				eiCount			= getExtraInfoCount (Day);
	EVENT 			ev;
	TEXT_EDIT 		te;
	WINDOW			Win;
	SCR_RECT		rc 				= {{4,15,235,116}};
	TIdleLoop		idl;
	
	idl.Task = TASK_AGENDA;
	D_AgendaViewMode = (D_AgendaViewMode | EDITOR_VIEW);
	
	if (!WinOpen (&Win, MakeWinRect (4, 17, 235, 116), WF_NOBORDER)) Ret = ER_MALLOC;
	else
	{	
		MustCloseWin = 1;
		Win.Flags &= ~WF_DIRTY;
		ScrRectFill (&rc, ScrRect, A_REVERSE);
	
		if (!TE_open (&te, &Win, MakeWinRect (5, 5, 226, 96 - 10*eiCount), *hBuf, 0, 0, 1)) Ret = ER_MALLOC;
	}
	
	if (Ret == ER_SUCCESS)
	{
		if (eiCount) DrawExtraInfo (Day, eiCount, NORMAL_VIEW);
		Cursor = CU_start ();
		Key = 0;
		TE_focus (&te);
		while ( Ret == ER_SUCCESS && (Key != KEY_ESC) && (Key != (KEY_2ND + KEY_ESC)) && !gr_Agenda->Exit )
		{
			IdleLoop (&idl, ILF_NOMENU, &Key, NULL, &ev);  
			TE_handleEvent (&te, &ev);

			if (Key == KEY_DIAMOND + KEY_ON) off();
			if (Key == KEY_DIAMOND + 'B') Ret = ShowBirthdays (Day);
		}

		if (Key == KEY_ESC + KEY_2ND) Ret = DONT_SAVE_DAY;
		else Ret = SAVE_DAY;

		TE_shrinkWrap (&te);
		CU_restore (Cursor);
	}
	
	if (MustCloseWin) WinRemove (&Win, 0);

	if (!gr_Agenda->Exit)
	{
		D_AgendaViewMode = (D_AgendaViewMode & ~EDITOR_VIEW);
		if ( (D_AgendaViewMode & MASK_VIEW) == NORMAL_VIEW) InitGridView();
		else initMonthView();
	}
		
	return (Ret);
}
	

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- If we press enter on a Day -------------------------------------------
// --------------------------------------------------------------------------

int EditDay (unsigned int Day)
{
	char			*buf 		= NULL;
	HANDLE			hBuf;
	unsigned long	Size;
	int				Ret 		= ER_SUCCESS;

	buf = ReadDay (Day, NULL);
	if (buf)
	{
		Size = strlen (buf)+1;
		HeapUnlock (gr_Agenda->DataFile.handle);
		hBuf = HeapAlloc (Size);
		HeapLock (gr_Agenda->DataFile.handle);
		ReDerefDataFile();
		if (hBuf == H_NULL) Ret = ER_MALLOC;
		if (Ret == ER_SUCCESS)
		{
			buf = ReadDay (Day, NULL);
			memcpy (HeapDeref (hBuf), buf, Size);
		}
	}
	else
	{
		hBuf = HeapAlloc (1);
		if (hBuf == H_NULL) Ret = ER_MALLOC;
		else poke (HeapDeref (hBuf), '\0');
	}
	
	if (Ret == ER_SUCCESS)
	{
		Ret = ShowEditor (&hBuf, Day);
	}
	
	if (Ret == ER_SUCCESS || Ret == SAVE_DAY || Ret == DONT_SAVE_DAY)
	{
		if (Ret == SAVE_DAY) Ret = WriteDay (Day, hBuf);
		Ret = ER_SUCCESS;
	}

	HeapFreeHNULL (&hBuf);
	
	return (Ret);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Prints the inversed full Datetext, the headline. ---------------------
// --------------------------------------------------------------------------

void PrintHead (TDate Date)
{
	char		Str[200];

	sprintf (Str, HEAD_STRING,	WEEK_DAYS[Date.WeekDay],
								Date.MonthDay, 
								MONTHS[Date.Month-1],
								Date.Year, Date.Week);
	PrintHeadBox (Str);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Prints the Date given by Date at (5,y) in fix style ------------------
// --------------------------------------------------------------------------

void PrintDate (int y, unsigned int Day)
{
	char	buf[20];
	
	getDateString (Day, NULL, buf, FLAG_USE_WEEKDAY, NULL);
	FontSetSys (F_6x8);
	DrawStr (5,y, buf, A_REPLACE);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Prints the preview of the day on which the cursor is -----------------
// --------------------------------------------------------------------------

void PrintHeadData (int y, unsigned int Day)
{
	char			*data 			= NULL;
	char			Disp [100];
	SCR_RECT		rc 				= {{70,y,235,y+7}};
	unsigned int	iconPos			= 225 +9;
	int				hashValue		= ReadDayWithExtraData (Day);
	int				showMDI			= 0;	// show MoreData icon
	unsigned int	nLetters;
	char			*pLF;
	
	ScrRectFill (&rc, ScrRect, A_REVERSE);

	if (hashValue & HASH_BIRTHDAY) BitmapPut (iconPos -= 9, y, &BirthDay, ScrRect, A_REPLACE);
	if (hashValue & HASH_PUBLIC_HDAY) BitmapPut (iconPos -= 9, y, &Cross, ScrRect, A_REPLACE);

	data = ReadDay (Day, NULL);
	if (data)
	{
		strncpy (Disp, data, 80);
		Disp[80] = 0;
		pLF = strchr (Disp, 0x0D);
		if (pLF)
		{
			*pLF = 0;
			showMDI = 1;
		}

		FontSetSys (F_4x6);
		nLetters = getNumLetters (Disp, iconPos - 70 -9);

		if (strlen (Disp) > nLetters) showMDI = 1;
		Disp[nLetters] = 0;		
		DrawStrXY (70, y+1, Disp, A_NORMAL);
		if (showMDI) BitmapPut (iconPos -= 9, y, &MoreData, ScrRect, A_REPLACE);
	}
	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Draws the inversed Dateline, in the Gridview -------------------------
// --------------------------------------------------------------------------

void DrawMainCursor (TOnOff State, unsigned Day)
{
	SCR_RECT	MainCursor; //= {{4,55,235,64}};
	
	if (State != gr_Agenda->MainCursor)
	{
		if (D_AgendaViewMode == NORMAL_VIEW)
		{
			MainCursor.l = 0x0437EB40;	// = {{4,55,235,64}};
			ScrRectFill (&MainCursor, ScrRect, A_XOR);
		}
		else
		{
			MainCursor.xy.y0 = 88;
			MainCursor.xy.y1 = 96;
			MainCursor.xy.x0 = 20 + (Day % 7)*31;
			MainCursor.xy.x1 = 49 + (Day %7)*31;
			ScrRectFill (&MainCursor, ScrRect, A_XOR);
		}			
		
		gr_Agenda->MainCursor = !gr_Agenda->MainCursor;
	}
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Scroll the entire Grid-view, calc the next Date, redraw the headline -
// --- redraw the cursor. ---------------------------------------------------
// --- int Up:	0: Down; !0 : Up --------------------------------------------
// --- Day:	Daynumber at cursorposition -------------------------------------
// --------------------------------------------------------------------------

void Scroll (int Up, unsigned int Day)
{
	char		DateArea [BITMAP_HDR_SIZE + 240*86/8];
	SCR_RECT	UpWin = {{0,30,239,115}};
	SCR_RECT	DnWin = {{0,17,239,102}};
	SCR_RECT	CuDown = {{4,38,235,47}};
	SCR_RECT	CuUp = {{4,25,235,34}};
	TDate		Date;
	
	if (Up)
	{
		BitmapGet (&UpWin,DateArea);
		PortSet (DateArea+BITMAP_HDR_SIZE, 239, 85);
		ScrRectFill (&CuUp, ScrRect, A_XOR);
		ScrRectFill (&CuDown, ScrRect, A_XOR);
		PortRestore();
		BitmapPut (0,17,DateArea,ScrRect, A_REPLACE);
		PrintDate (108,Day+4);
		PrintHeadData (108,Day+4);
	}
	else
	{
		BitmapGet (&DnWin,DateArea);
		PortSet (DateArea+BITMAP_HDR_SIZE, 239, 85);
		ScrRectFill (&CuDown, ScrRect, A_XOR);
		ScrRectFill (&CuUp, ScrRect, A_XOR);
		PortRestore();
		BitmapPut (0,30,DateArea,ScrRect, A_REPLACE);
		PrintDate (17,Day-3);
		PrintHeadData (17,Day-3);
	}
	GetDateEx (Day, &Date);
	PrintHead (Date);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Fills the Grid with recalculation of all dates; redraw cursor --------
// --- and head. ------------------------------------------------------------
// --------------------------------------------------------------------------

void FillGrid (unsigned int Day)
{
	unsigned int	i;
	unsigned int	First = Day-3;
	unsigned int	Last  = Day+5;
	int				y = 17; 
	TDate			Date;
	LCD_BUFFER		Screen;
	
	LCD_save (Screen);
	PortSet (Screen, 239,127);
	GetDateEx (Day, &Date);
	PrintHead (Date);
	DrawMainCursor(OFF, 0);
	for (i=First; i<Last; i++)
	{
		PrintDate (y, i);
		PrintHeadData (y, i);
		y += 13;
	}
	DrawMainCursor(ON, 0);
	PortRestore();
	LCD_restore(Screen);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int AgendaOnMenuOpen (void *ptr1, void *ptr2, int *Tag1, int *Tag2)
{
	gr_Agenda->DataFile.Header->LastAgendaDay = *((unsigned int *)ptr1);
	DrawMainCursor (OFF, *((unsigned int *)ptr1));
	
	return 0;
}

int AgendaOnMenuClose (void *ptr1, void *ptr2, int Tag1, int Tag2, int DidWhat, int Feedback) 
{
	unsigned int		Day;
	int					reFill	= 0;

	Day = gr_Agenda->DataFile.Header->LastAgendaDay;

	if ( (DidWhat == RET_MM_RESTORE_BACKUP) && (Feedback == ER_SUCCESS) )
	{
		reFill = 1;
		*(unsigned int *)ptr1 = Day;
	}

	if ( (DidWhat == RET_MM_CONFIG) )
	{
		reFill = 1;
		fillHashTable (Day, 1);
	}
	
	if (reFill)
	{
		if (D_AgendaViewMode == NORMAL_VIEW) FillGrid (Day);
		else fillMonthView (Day, Day, SCROLL_NOT);
	}
	else DrawMainCursor (ON, Day);
	return 0;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Show Gridview. Get keystrokes; is the maincomandwindow ---------------
// --- return:		actual Day ----------------------------------------------
// --------------------------------------------------------------------------

int ShowGridView (unsigned int *Day)
{
	int				Ret					= ER_SUCCESS;
	int				key					= 0;
	int				Repaint;
	unsigned int	DayBefore;
	TIdleLoop		idl;
	
	idl.BeforeMenuHook = &AgendaOnMenuOpen;
	idl.AfterMenuHook = &AgendaOnMenuClose;
	idl.Task = TASK_AGENDA;
	
	InitGridView();
	FillGrid (*Day);
	
	while ( (key != KEY_ESC) && !gr_Agenda->Exit && (Ret == ER_SUCCESS) )
	{
		Repaint = 0;
		FillGrid (*Day);

		while ( (key != KEY_ESC) && !Repaint && !gr_Agenda->Exit && (Ret == ER_SUCCESS) )
		{
			idl.Ptr1 = Day;
			IdleLoop (&idl, 0, &key, NULL, NULL);

			if ( (key == KEY_UP) || (key == KEY_DOWN) )
			{
				if (key == KEY_UP && *Day > MIN_DAY) Scroll(0, --(*Day));
				if (key == KEY_DOWN && *Day < MAX_DAY) Scroll (1, ++(*Day));
			}
			else
			{
				Repaint = 1;
				DayBefore = *Day;
				if (key == (KEY_2ND + KEY_UP)) *Day -= 7;
				if (key == (KEY_DIAMOND + KEY_UP)) *Day -=14;
				if (key == (KEY_SHIFT + KEY_UP)) *Day -=28;
	
				if (key == (KEY_2ND + KEY_DOWN)) *Day += 7;
				if (key == (KEY_DIAMOND + KEY_DOWN)) *Day += 14;
				if (key == (KEY_SHIFT + KEY_DOWN)) *Day += 28;
				
				CheckValidDay (Day);
	
				if (*Day == DayBefore) Repaint = 0;

				if ( IsAgendaEventKey (key) ) Ret = MenuAgenda (Day, &Repaint, key, NULL); 
			
				// --- if MenuGotoNextEntry() and no entry
				if (Ret == ER_NOENTRY) Ret = ER_SUCCESS;
				
				if (Ret == CANCEL) Ret = ER_SUCCESS;

				CheckValidDay (Day);

			}

			fillHashTable (*Day, 0);
		}
		
	}
	
	return (Ret);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Blank Screen, draw border and lines of GridView. ---------------------
// --------------------------------------------------------------------------

void InitGridView ()
{
	int i;
	
	EmptyScreenClient();
	DrawLine (65,15,65,116,A_NORMAL);
	FontSetSys (F_6x8);
	for (i=1; i<9; i++)
		DrawLine (3,i*13+1,236,i*13+1, A_NORMAL);
	gr_Agenda->MainCursor = OFF;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void displayMonthViewText (unsigned int Day)
{
	char			*data;
	char			buf[50];
	char			*CR;
	int				y			= 0;
	int				hashValue	= ReadDayWithExtraData (Day);
	int				iconPos		= 226 +9;
	int				eiCount		= getExtraInfoCount (Day);
	
	ScrRectFill (&(SCR_RECT){{4,15,235,63}}, ScrRect, A_REVERSE);
	data = ReadDay (Day, NULL);
	FontSetSys (F_6x8);

	if (data)
	{
		while (y < 5 && *data != 0)
		{
			memcpy (buf, data, 37);
			buf[37] = 0;
			CR = strchr (buf, 13);
			if (CR != NULL)
			{
				*CR = 0;
				data++;
			}
			data += strlen(buf);
			
			DrawStr (8, y*9+17, buf, A_NORMAL);
			y++;
		}
	
	}

	if (y < (6 - eiCount) && eiCount) DrawExtraInfo (Day, eiCount, SECOND_VIEW); 
	
	if (*data != 0 || (eiCount && y >= (6 - eiCount)))
	{
		DrawFilledRect (204, 14, 236, 30);
		if (*data != 0) BitmapPut (iconPos -= 9, 19, &MoreData, ScrRect, A_REPLACE);
		if (hashValue & HASH_BIRTHDAY) BitmapPut (iconPos -= 9, 19, &BirthDay, ScrRect, A_REPLACE);
		if (hashValue & HASH_PUBLIC_HDAY) BitmapPut (iconPos -= 9, 19, &Cross, ScrRect, A_REPLACE);
	}
	
}

void fillMonthView (unsigned int dayBefore, unsigned int Day, int scroll)
{
	unsigned int	hashValue;
	unsigned int	refDay		= Day - (Day % 7) - 7;	// Day of first Monday in list
	int				start		= 0;
	int				stop		= 28;
	int				i;
	int				y, x;
	TDate			date;
	LCD_BUFFER		Screen;
	char			ScrollArea [BITMAP_HDR_SIZE + 240*29/8];
	
	LCD_save (Screen);
	PortSet (Screen, 239,127);
	DrawMainCursor (OFF, dayBefore);

	displayMonthViewText (Day);	

	if (Day / 7 == dayBefore / 7) scroll = SCROLL_NOT;
	if (scroll == SCROLL_UP)
	{
		BitmapGet (&(SCR_RECT){{0, 78, 239, 106}}, ScrollArea);
		start = 0;
		stop = 7;
	}
	else if (scroll == SCROLL_DOWN)
	{
		BitmapGet (&(SCR_RECT){{0, 88, 239, 116}}, ScrollArea);
		start = 21;
		stop = 28;
	}

	FontSetSys (F_4x6);
	for (i = start; i < stop; i++)
	{
		y = (i / 7)*10 + 80;
		x = 37 + (i % 7)*31;

		GetDate (refDay + i, &date);
		if ( ((refDay + i) % 7) == 0)
		{
			// --- display week number
			printf_xy (6, y, "%02d", GetWeek (refDay + i));
		}

		// --- display day numbers
		printf_xy (x - 14, y, "%02d", date.MonthDay);
		
		// --- display icons
		hashValue = ReadDayWithExtraData (refDay + i);
		if (hashValue == HASH_BIRTHDAY)
			BitmapPut (x, y, &smallBirthDay, ScrRect, A_REPLACE); 
		else if ( (hashValue <= HASH_PUBLIC_HDAY) && hashValue != 0)
			BitmapPut (x, y, &smallCross, ScrRect, A_REPLACE); 
		else if (hashValue != 0)
			ScrRectFill (&(SCR_RECT){{x, y, x + 7, y+4}}, ScrRect, A_NORMAL);
		else
			ScrRectFill (&(SCR_RECT){{x, y, x + 7, y+4}}, ScrRect, A_REVERSE);

	}

	if (scroll == SCROLL_UP) BitmapPut (0, 88, ScrollArea, ScrRect, A_REPLACE);
	else if (scroll == SCROLL_DOWN)	BitmapPut (0, 78, ScrollArea, ScrRect, A_REPLACE);

	GetDateEx (Day, &date);
	PrintHead (date);

	DrawMainCursor (ON, Day);
	PortRestore();
	LCD_restore(Screen);
}


void ScrollHoriz (unsigned int dayBefore, unsigned int Day)
{
	TDate			date;
	LCD_BUFFER		Screen;

	if ( (dayBefore % 7) == 6 && (Day % 7) == 0)
	{
		fillMonthView (dayBefore, Day, SCROLL_DOWN); 
	}
	else if ((dayBefore % 7) == 0 && (Day % 7) == 6)
	{
		fillMonthView (dayBefore, Day, SCROLL_UP); 
	}
	else
	{
		LCD_save (Screen);
		PortSet (Screen, 239,127);
	
		displayMonthViewText (Day);	
		DrawMainCursor (OFF, dayBefore);
		DrawMainCursor (ON, Day);
	
		GetDateEx (Day, &date);
		PrintHead (date);
		
		PortRestore();
		LCD_restore(Screen);
	}
}

int ShowMonthView (unsigned int *Day)
{
	int				Ret					= ER_SUCCESS;
	int				key					= 0;
	int				Repaint				= 1;
	unsigned int	dayBefore;
	TIdleLoop		idl;
	int				scroll;
	
	idl.BeforeMenuHook = &AgendaOnMenuOpen;
	idl.AfterMenuHook = &AgendaOnMenuClose;
	idl.Task = TASK_AGENDA;
	
	initMonthView ();

	while ( (key != KEY_ESC) && !gr_Agenda->Exit && (Ret == ER_SUCCESS) )
	{
		if (Repaint) fillMonthView (*Day, *Day, SCROLL_NOT);

		Repaint = 0;
		scroll = SCROLL_NOT;
		idl.Ptr1 = Day;
		IdleLoop (&idl, 0, &key, NULL, NULL);
		
		if (key == KEY_RIGHT && *Day < MAX_DAY)
		{
			(*Day)++;
			ScrollHoriz (*Day -1, *Day);
		}
		if (key == KEY_LEFT && *Day > MIN_DAY)
		{
			(*Day)--;
			ScrollHoriz (*Day +1, *Day);
		}

		dayBefore = *Day;
		if (key == KEY_DOWN)
		{
			*Day += 7;
			scroll = SCROLL_DOWN;
		}
		if (key == KEY_UP)
		{
			*Day -= 7;
			scroll = SCROLL_UP;
		}
		
		if ( IsAgendaEventKey (key) ) Ret = MenuAgenda (Day, &Repaint, key, NULL); 
		CheckValidDay (Day);

		fillHashTable (*Day, 0);

		if (*Day != dayBefore && !Repaint)
		{
			fillMonthView (dayBefore, *Day, scroll);
		}

		// --- if MenuGotoNextEntry() and no entry
		if (Ret == ER_NOENTRY) Ret = ER_SUCCESS;
				
		if (Ret == CANCEL) Ret = ER_SUCCESS;
	}

	return (Ret);
}

void initMonthView (void)
{
	int		i;

	EmptyScreenClient();
	DrawLine (3, 64, 236, 64, A_THICK1);	// horizontal double thick line
	DrawLine (18, 64, 18, 116, A_NORMAL);	// vertical divider between weeknr and days
	DrawLine (19, 76, 236, 76, A_NORMAL);	// horizontal divider between day names and days

	FontSetSys (F_6x8);
	for (i = 0; i < 8; i++)
	{
		// --- 8 vertical lines
		DrawLine (19+i*31, 65, 19+i*31, 116, A_NORMAL);	
		// --- 5 horizontal lines
		if (i < 5) DrawLine (3, i*10+77, 236, i*10+77, A_NORMAL);
		// --- Draw week names
		if (i < 7) DrawStr (27+i*31, 67, WEEK_DAYS[i], A_NORMAL);
	}
	
	FontSetSys (F_4x6);
	DrawStr (5, 69, TEXT_CW, A_NORMAL);		// TEXT_CW = "CW"
	gr_Agenda->MainCursor = OFF;

}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int AgendaView (int task)
{
	int				Ret 	= ER_SUCCESS;
	unsigned int	Day;
	TDate			date;
	int				mode;
	
	if (!gr_Agenda->TaskQueue.keepTask)
	{
		gr_Agenda->hashSet.year = 0;
		EmptyScreenClient();
	}

	if (gr_Agenda->TaskQueue.prm0 != TASK_NO_MESSAGE)
	{
		Day = gr_Agenda->TaskQueue.prm0;
		mode = gr_Agenda->TaskQueue.NextMode;
	}
	else
	{
		Day = ReadDayNumber ();
		mode = D_AgendaViewMode;
	}
	
	resetTaskQueueParams();
	CheckValidDay (&Day);
	Ret = fillHashTable (Day, 0);

	// --- if we should direct jump to the editor first...
	if ( (Ret == ER_SUCCESS) && (mode & EDITOR_VIEW))
	{
		GetDateEx (Day, &date);
		FontSetSys (F_6x8);
		PrintHead (date);

		Ret = EditDay (Day);
	}

	if (Ret == ER_SUCCESS && !gr_Agenda->Exit)
	{
		do
		{	
			if (D_AgendaViewMode == NORMAL_VIEW) Ret = ShowGridView (&Day);
			if (D_AgendaViewMode == SECOND_VIEW) Ret = ShowMonthView (&Day);
		}
		while (Ret == ER_CHANGE_VIEW);
			
		gr_Agenda->DataFile.Header->LastAgendaDay = Day;
	}
		
	return Ret;	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
