// C Source File

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AgndConv V.E oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Date:         November 2003 ooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------


#include <tigcclib.h>

#include "BuildOptions.h"
#include "Agndconv.h"
#include "consts.h"
#include "V3.h"
#include "V4.h"
#include "VE.h"

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int Update4toE (HANDLE handle)
{
	int					ret			= ER_SUCCESS;
	char				*source		= HeapDeref (handle);
	TDataFileV4			*file		= (TDataFileV4 *)source;
	TAgendaHeaderV4		*header;
	TAgendaHeaderVE		*headerNew;
	unsigned int		size;
	TEntryHeaderV4		*headerEntry;
	char				*birthday;
	
	HeapLock (handle);
	if ( MsgBox ((char *)"Conversion", (char *)"Update agnddata version 4 to E?\0The version of agnddata is now\0counted with letters due to\0confusion with the version of\0Agenda itself.\0", BT_YES, BT_NO) == OK )
	{
		// --- update version identifier
		file->Version = 0xFF05;

		// --- decode file, if necessery
		if (file->Flags & FLAG_ENCODED)
		{
			file->Flags = file->Flags & ~FLAG_ENCODED;
			EncodeData (source + 74, ENCODE_KEY_D, peek_w (source)+2 -N_TERM_BYTES - 74 );
		}


		// --- Agenda
		// --- convertig to new AgendaHeader, which is now compatible to EntryHeader
		// --- introcude new Flags, which allow identification of task in each entry
		// --- clear shorcut
		header = (TAgendaHeaderV4 *)(source + file->AgendaPtr);
		while (header->Flags != 0)
		{
			header->Flags = FLAG_ET_AGENDA | FLAG_ET_ENTRY;
			size = header->Size;
			headerNew = (TAgendaHeaderVE *)header;
			headerNew->Day = header->Day;
			headerNew->Shortcut = 0;
			headerNew->Size = size;
			header = (TAgendaHeaderV4 *)( ((char *)header) + size);
		}


		// --- Addressbook		
		// --- replace every '/' by a ';'. (date separator)
		// --- introcude new Flags, which allow identification of task in each entry
		// --- clear shorcut
		headerEntry = (TEntryHeaderV4 *)(source + file->AddressbookPtr);
		while (headerEntry->Flags != 0)
		{
			headerEntry->Shortcut = 0;
			if (headerEntry->Flags == 1) headerEntry->Flags = FLAG_ET_ADDRESSBOOK | FLAG_ET_GROUP;
			if (headerEntry->Flags == 2)
			{
				headerEntry->Flags = FLAG_ET_ADDRESSBOOK | FLAG_ET_ENTRY;
				birthday = GetStrFromListEx (&headerEntry->Data, 7);
				strreplce (birthday, '/', ';');
			}
			
			headerEntry = (TEntryHeaderV4 *)( ((char *)headerEntry) + headerEntry->Size);
		}


		// --- Notepad		
		// --- introcude new Flags, which allow identification of task in each entry
		// --- clear shorcut
		headerEntry = (TEntryHeaderV4 *)(source + file->NotesPtr);
		while (headerEntry->Flags != 0)
		{
			headerEntry->Shortcut = 0;
			// --- Group
			if (headerEntry->Flags == 1) headerEntry->Flags = FLAG_ET_NOTES | FLAG_ET_GROUP;
			// --- Entry
			if (headerEntry->Flags == 2) headerEntry->Flags = FLAG_ET_NOTES | FLAG_ET_ENTRY;
			headerEntry = (TEntryHeaderV4 *)( ((char *)headerEntry) + headerEntry->Size);
		}


		// --- Weblinks		
		// --- introcude new Flags, which allow identification of task in each entry
		// --- clear shorcut
		headerEntry = (TEntryHeaderV4 *)(source + file->INetLinksPtr);
		while (headerEntry->Flags != 0)
		{
			headerEntry->Shortcut = 0;
			// --- Group
			if (headerEntry->Flags == 1) headerEntry->Flags = FLAG_ET_LINKS | FLAG_ET_GROUP;
			// --- Entry
			if (headerEntry->Flags == 2) headerEntry->Flags = FLAG_ET_LINKS | FLAG_ET_ENTRY;
			headerEntry = (TEntryHeaderV4 *)( ((char *)headerEntry) + headerEntry->Size);
		}


		// --- reencode file
		#ifndef DONT_ENCODE_FILE
		if (!(file->Flags & FLAG_ENCODED))
		{
			file->Flags = file->Flags | FLAG_ENCODED;
			EncodeData (source + 74, ENCODE_KEY_D, peek_w (source)+2 - N_TERM_BYTES - 74 );
		}
		#else
		#warning "File won't be encoded at the end!"
		#endif

		MsgBox ((char *)"Conversion", (char *)"Conversion completed successfully!", BT_OK, BT_NONE)		;
	}
	else ret = CANCEL;
	
	HeapUnlock (handle);
		
	return ret;
}

